@@ -1,7 +1,7 @@
 ;; -*- mode: emacs-lisp -*-
 ;; mew-shimbun.el --- View shimbun contents with Mew
 
-;; Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2010
+;; Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2010, 2016
 ;; TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 
 ;; Author: TSUCHIYA Masatoshi <tsuchiya@namazu.org>
@@ -1023,9 +1023,12 @@ If called with '\\[universal-argument]', re-retrieve messages in the region."
 	(forward-line)
 	(mew-header-goto-next)
 	(delete-region beg (point)))
-      (md5 (string-as-unibyte
-	    (mew-buffer-substring (point-min)
-				  (min (point-max) (+ (point-min) 4096))))
+      (md5 (static-if (featurep 'emacs)
+	       (string-make-unibyte
+		(mew-buffer-substring (point-min)
+				      (min (point-max) (+ (point-min) 4096))))
+	     (mew-buffer-substring (point-min)
+				   (min (point-max) (+ (point-min) 4096))))
 	   nil nil 'binary))))
 
 (defvar mew-shimbun-touch-folder-p
