@@ -1,6 +1,6 @@
 ;;; w3m-proc.el --- Functions and macros to control sub-processes
 
-;; Copyright (C) 2001-2005, 2007-2010, 2012
+;; Copyright (C) 2001-2005, 2007-2010, 2012, 2013, 2016
 ;; TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 
 ;; Authors: TSUCHIYA Masatoshi <tsuchiya@namazu.org>,
@@ -60,11 +60,6 @@
   (defvar w3m-work-buffer-list)
   (autoload 'w3m-idle-images-show-unqueue "w3m"))
 
-;; Silence the Emacs's byte-compiler that says ``might not be defined''.
-(eval-when-compile
-  (defun w3m-decode-coding-string-with-priority (str coding)
-    ()))
-
 (defvar w3m-process-inhibit-quit t
   "`w3m-process-sentinel' binds `inhibit-quit' according to this variable.")
 (defvar w3m-process-timeout 300
@@ -467,15 +462,15 @@ evaluated in a temporary buffer."
 		    (w3m-get-buffer-create
 		     (generate-new-buffer-name w3m-work-buffer-name)))
 		   (,current-buffer (current-buffer)))
-       (labels ((,post-body (,var)
-			    (when (buffer-name ,temp-buffer)
-			      (set-buffer ,temp-buffer))
-			    ,@body)
-		(,post-handler (,var)
-			       (w3m-kill-buffer ,temp-buffer)
-			       (when (buffer-name ,current-buffer)
-				 (set-buffer ,current-buffer))
-			       ,var))
+       (w3m-labels ((,post-body (,var)
+				(when (buffer-name ,temp-buffer)
+				  (set-buffer ,temp-buffer))
+				,@body)
+		    (,post-handler (,var)
+				   (w3m-kill-buffer ,temp-buffer)
+				   (when (buffer-name ,current-buffer)
+				     (set-buffer ,current-buffer))
+				   ,var))
 	 (let ((,var (let ((handler
 			    (cons #',post-body
 				  (cons #',post-handler handler))))
@@ -519,7 +514,9 @@ evaluated in a temporary buffer."
     (setq w3m-process-exit-status nil))
    (t
     (setq w3m-process-exit-status
-	  (string-as-multibyte (format "%s" exit-status)))
+	  (w3m-static-if (featurep 'emacs)
+	      (string-make-multibyte (format "%s" exit-status))
+	    (format "%s" exit-status)))
     nil)))
 
 (defvar w3m-process-background nil
