@@ -1,6 +1,6 @@
 ;;; w3m-session.el --- Functions to operate session of w3m -*- coding: iso-2022-7bit; -*-
 
-;; Copyright (C) 2001-2003, 2005-2012 TSUCHIYA Masatoshi <tsuchiya@namazu.org>
+;; Copyright (C) 2001-2003, 2005-2013 TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 
 ;; Author: Hideyuki SHIRAI <shirai@meadowy.org>
 ;; Keywords: w3m, WWW, hypermedia
@@ -731,7 +731,9 @@ Sorry, an error found in \"%s\"; may we remove it? "
 	     ((assoc title sessions)
 	      (when (y-or-n-p (format "\"%s\" is exist. Overwrite? " title))
 		(setq overwrite t)
-		(throw 'loop t))))
+		(throw 'loop t)))
+	     (t
+	      (throw 'loop t)))
 	    (setq prompt "Again New session title: ")))
 	(when overwrite
 	  (setq sessions (delete (assoc title sessions) sessions)))
