@@ -1,6 +1,6 @@
 ;;; w3m-util.el --- Utility macros and functions for emacs-w3m
 
-;; Copyright (C) 2001-2014 TSUCHIYA Masatoshi <tsuchiya@namazu.org>
+;; Copyright (C) 2001-2014, 2016 TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 
 ;; Authors: TSUCHIYA Masatoshi <tsuchiya@namazu.org>,
 ;;          Shun-ichi GOTO     <gotoh@taiyo.co.jp>,
@@ -516,12 +516,12 @@ An argument of nil means kill the current buffer."
 	   (setq x (cons x
 			 (if (match-beginning 1)
 			     (string-to-number (match-string 2 x))
-			   1))))
+			   0))))
       (if (string-match "\\*w3m\\*\\(<\\([0-9]+\\)>\\)?\\'" y)
 	  (< (cdr x)
 	     (if (match-beginning 1)
 		 (string-to-number (match-string 2 y))
-	       1))
+	       0))
 	(string< (car x) y))
     (string< x y)))
 
@@ -758,7 +758,7 @@ objects will not be deleted:
 	(set-buffer buffer)
 	(while windows
 	  (setq window (pop windows)
-		frame (window-frame window))
+		frame (and (window-live-p window) (window-frame window)))
 	  (when (and frame
 		     (not (eq frame exception)))
 	    (setq one-window-p
@@ -1364,12 +1364,12 @@ pairs from PLIST whose value is nil."
 (defmacro w3m-insert-string (string)
   "Insert STRING at point without conversions in either case the
 multibyteness of the buffer."
-  (if (and (fboundp 'string-as-multibyte)
-	   (subrp (symbol-function 'string-as-multibyte)))
+  (if (and (fboundp 'string-make-multibyte)
+	   (subrp (symbol-function 'string-make-multibyte)))
       `(let ((string ,string))
 	 (insert (if enable-multibyte-characters
-		     (string-as-multibyte string)
-		   (string-as-unibyte string))))
+		     (string-make-multibyte string)
+		   (string-make-unibyte string))))
     `(insert ,string)))
 
 (defun w3m-custom-hook-initialize (symbol value)
@@ -1614,6 +1614,7 @@ the function cell of FUNCs rather than their value cell.
 	 ,@body)
     `(flet ,bindings ,@body)))
 (put 'w3m-flet 'lisp-indent-function 1)
+(def-edebug-spec w3m-flet ((&rest (sexp sexp &rest form)) &rest form))
 
 (defmacro w3m-labels (bindings &rest body)
   "Make temporary function bindings.
@@ -1626,6 +1627,7 @@ rather than relying on `lexical-binding'.
   `(,(progn (require 'cl) (if (fboundp 'cl-labels) 'cl-labels 'labels))
     ,bindings ,@body))
 (put 'w3m-labels 'lisp-indent-function 1)
+(def-edebug-spec w3m-labels ((&rest (sexp sexp &rest form)) &rest form))
 
 (eval-when-compile (require 'wid-edit))
 (defun w3m-widget-type-convert-widget (widget)
